<?php
namespace ReyCore;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Admin {

	public function __construct(){

		add_action('init', [$this, 'init']);

		new \ReyCore\Libs\IgTokenManager;
	}

	public function init(){
		add_action( 'admin_init', [$this, 'admin_init']);
		add_action( 'admin_head', [$this, 'admin_bar_css'] );
		add_action( 'wp_head', [$this, 'admin_bar_css'] );
		add_action( 'admin_menu', [$this, 'admin_menu'] );
		add_action( 'admin_bar_menu', [$this, 'admin_bar_links'], 200 );
		add_action( 'admin_enqueue_scripts', [$this, 'register_admin_scripts'], 5);
		add_action( 'admin_enqueue_scripts', [$this, 'enqueue_admin_scripts']);
		add_action( 'wp_enqueue_scripts', [$this, 'enqueue_scripts']);
	}

	public function admin_init(){

		if( ! class_exists('\ACF') ){
			return false;
		}

		if( reycore__acf_get_field('rey_widgets_blocks_layout', REY_CORE_THEME_NAME) === true ){
			return;
		}

		// Disables the block editor from managing widgets in the Gutenberg plugin.
		add_filter( 'gutenberg_use_widgets_block_editor', '__return_false' );
		// Disables the block editor from managing widgets.
		add_filter( 'use_widgets_block_editor', '__return_false' );
	}

	public function admin_bar_css() {

		if ( ! is_admin_bar_showing() ) {
			return;
		} ?>

		<style type="text/css">
			#wpadminbar .rey-abQuickMenu .rey-abQuickMenu-logo {
				/* width: 45px; */
				height: inherit;
				line-height: inherit;
				display: flex;
			}
			#wpadminbar .rey-abQuickMenu .rey-abQuickMenu-logo img {
				display: block;
				width: 100%;
				max-width: 32px;
				opacity: .5;
				margin: 5px auto 0;
			}
			.rey-abQuickMenu:hover .rey-abQuickMenu-logo img {
				opacity: .7;
			}
			#wpadminbar .rey-abQuickMenu .rey-abQuickMenu-top {
				margin-top: 10px;
			}
			#wpadminbar .rey-abQuickMenu .rey-abQuickMenu-notices {
				margin-top: 15px;
			}
		</style>

		<?php
		wp_enqueue_style('reycore-frontend-admin');

	}

	public function admin_menu(){

		if( apply_filters('reycore/admin_menu', reycore__get_props('admin_menu')) ){
			return;
		}

		remove_menu_page( reycore__get_dashboard_page_id() );

	}

	/**
	 * Admin bar links
	 *
	 * @since 1.4.0
	 **/
	function admin_bar_links( $admin_bar ) {

		if( ! current_user_can('administrator') ){
			return;
		}

		if( ! apply_filters('reycore/admin_bar_menu', reycore__get_props('admin_bar_menu') ) ){
			return;
		}

		if( !($dashboard_id = reycore__get_dashboard_page_id()) ){
			return;
		}

		$parent_content = reycore__get_props('theme_title');

		if( $parent_custom_text = reycore__get_props('button_text') ){
			$parent_content = $parent_custom_text;
		}

		if( ($parent_icon = reycore__get_props('button_icon')) ){
			$parent_content = sprintf('<img src="%1$s" alt="%2$s"><span class="screen-reader-text">%2$s - Quick Menu</span>', $parent_icon, reycore__get_props('theme_title') );
		}

		$nodes['main'] = [
			'title'  => apply_filters('reycore/admin_bar_menu/html', sprintf('<span class="rey-abQuickMenu-logo">%s</span>', $parent_content)),
			'href'  => add_query_arg([
				'page' => $dashboard_id
				], admin_url( 'admin.php' )
			),
			'meta_title' => esc_html__( 'Rey - Quick Menu', 'rey-core' ),
			'class' => 'rey-abQuickMenu'
		];

		$nodes['dashboard'] = [
			'title'  => esc_html__( 'Dashboard', 'rey-core' ),
			'href'  => add_query_arg([
				'page' => $dashboard_id
				], admin_url( 'admin.php' )
			),
			'top' => true,
		];

		if( class_exists('\ReyCore\Elementor\GlobalSections') ):
			$nodes['gs'] = [
				'title'  => esc_html__( 'Global Sections', 'rey-core' ),
				'href'  => add_query_arg([
					'post_type' => \ReyCore\Elementor\GlobalSections::POST_TYPE
					], admin_url( 'edit.php' )
				),
			];
		endif;

		$nodes['settings'] = [
			'title'  => esc_html__( 'Theme Settings', 'rey-core' ),
			'href'  => add_query_arg([
				'page' => REY_CORE_THEME_NAME . '-settings',
				], admin_url( 'admin.php' )
			),
			'meta_title' => esc_html__( 'Generic theme settings.', 'rey-core' ),
		];

		$customize_url = urlencode( remove_query_arg( wp_removable_query_args(), wp_unslash( $_SERVER['REQUEST_URI'] ) ) );

		$nodes['customize'] = [
			'title'  => esc_html__( 'Customize Options', 'rey-core' ),
			'href'  => add_query_arg([
				'url' => $customize_url,
				], admin_url( 'customize.php' )
			),
			'meta_title' => esc_html__( 'Customize Options', 'rey-core' ),
			'top' => true,
			'nodes' => [
				[
					'title' => esc_html__( 'General settings', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[panel]' => \ReyCore\Customizer\Options\General::get_id()
						], admin_url( 'customize.php' )
					),
				],
				[
					'title' => esc_html__( 'Header', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[panel]' => \ReyCore\Customizer\Options\Header::get_id()
						], admin_url( 'customize.php' )
					),
					'meta_title' => esc_html__( 'Customize the site header.', 'rey-core' ),
					'top' => true
				],
				[
					'title' => esc_html__( 'Footer', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[panel]' => \ReyCore\Customizer\Options\Footer::get_id()
						], admin_url( 'customize.php' )
					),
					'meta_title' => esc_html__( 'Customize the footer.', 'rey-core' ),
				],
				[
					'title' => esc_html__( 'Page Cover', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[panel]' => \ReyCore\Customizer\Options\Cover::get_id()
						], admin_url( 'customize.php' )
					),
					'meta_title' => esc_html__( 'Customize the page covers (page headers).', 'rey-core' ),
				],
				[
					'title' => esc_html__( 'Blog', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[section]' => \ReyCore\Customizer\Options\General\BlogPage::get_id()
						], admin_url( 'customize.php' )
					),
					'meta_title' => esc_html__( 'Customize the site blog.', 'rey-core' ),
				],

			]
		];

		if( class_exists('\WooCommerce') ):

			$nodes['customize']['nodes'][] = [
				'title' => esc_html__( 'Woo > Catalog > Layout', 'rey-core' ),
				'href'  => add_query_arg([
					'url' => $customize_url,
					'autofocus[section]' => \ReyCore\Customizer\Options\Woocommerce\CatalogGrid::get_id()
					], admin_url( 'customize.php' )
				),
				'meta_title' => esc_html__( 'Customize the looks of the product catalog (eg: categories pages)', 'rey-core' ),
				'top' => true,
			];

			$nodes['customize']['nodes'][] = [
					'title' => esc_html__( 'Woo > Catalog > Components', 'rey-core' ),
					'href'  => add_query_arg([
						'url' => $customize_url,
						'autofocus[section]' => \ReyCore\Customizer\Options\Woocommerce\CatalogGridComponents::get_id()
						], admin_url( 'customize.php' )
					),
					'meta_title' => esc_html__( 'Enable or disable components in product catalog (categories).', 'rey-core' ),
				];

			$nodes['customize']['nodes'][] = [
				'title'  => esc_html__( 'Woo > Catalog > Misc.', 'rey-core' ),
				'href'  => add_query_arg([
					'url' => $customize_url,
					'autofocus[section]' => \ReyCore\Customizer\Options\Woocommerce\CatalogMisc::get_id()
					], admin_url( 'customize.php' )
				),
				'meta_title' => esc_html__( 'Various options', 'rey-core' ),
			];

			$nodes['customize']['nodes'][] = [
				'title'  => esc_html__( 'Woo > Product Page > Layout', 'rey-core' ),
				'href'  => add_query_arg([
					'url' => $customize_url,
					'autofocus[section]' => \ReyCore\Customizer\Options\Woocommerce\ProductPageLayout::get_id()
					], admin_url( 'customize.php' )
				),
				'meta_title' => esc_html__( 'Customize the looks of the product page.', 'rey-core' ),
				'top' => true,
			];

			$nodes['customize']['nodes'][] = [
				'title'  => esc_html__( 'Woo > Product Page > Components', 'rey-core' ),
				'href'  => add_query_arg([
					'url' => $customize_url,
					'autofocus[section]' => \ReyCore\Customizer\Options\Woocommerce\ProductPageComponents::get_id()
					], admin_url( 'customize.php' )
				),
				'meta_title' => esc_html__( 'Enable or disable components in product page.', 'rey-core' ),
			];

		endif;

		$nodes['css'] = [
			'title'  => esc_html__( 'Custom CSS', 'rey-core' ),
			'href'  => add_query_arg([
				'url' => $customize_url,
				'autofocus[section]' => 'custom_css'
				], admin_url( 'customize.php' )
			),
			'meta_title' => esc_html__( 'Add additional CSS.', 'rey-core' ),
		];

		if ( current_user_can( 'administrator' ) && class_exists('\ReyCore\Customizer\Styles') && \ReyCore\Customizer\Styles::css_cache_is_enabled() ) {
			$nodes['refresh_css'] = [
				'title'  => esc_html__( 'Refresh dynamic CSS', 'rey-core' ),
				'href'  => '#',
				'meta_title' => esc_html__( 'Refresh the dynamic CSS generated in Customizer.', 'rey-core' ),
				'class' => 'qm-refresh-css',
			];
		}

		if ( current_user_can( 'administrator' ) && function_exists('reyCoreAssets')) {
			if( ($asset_settings = reyCoreAssets()->settings) && ! $asset_settings['load_natural_css'] ){
				if( $asset_settings['store_data'] || $asset_settings['save_css'] || $asset_settings['save_js'] ){

					$href = '?clear_assets=2';
					$id = get_queried_object_id();

					if( is_singular() || is_home() || is_front_page() || (class_exists('\WooCommerce') && is_shop()) ){
						$href .= '&type=post&id='. $id;
					}
					else if(is_tax() || is_archive()){
						$href .= '&type=term&id='. $id;
					}

					$nodes['refresh_assets'] = [
						'title'  => esc_html__( 'Refresh Assets', 'rey-core' ),
						'href'  => $href,
						'meta_title' => esc_html__( 'Refresh the combined and minified assets.', 'rey-core' ),
						'class' => 'qm-refresh-assets',
					];

				}
			}
		}

		if( reycore__get_props('kb_links') ){

			$nodes['help'] = [
				'title'  => esc_html__( 'Help KB', 'rey-core' ),
				'href'  => reycore__support_url(),
				'meta_title' => esc_html__( 'Get help online.', 'rey-core' ),
				'new' => true,
				'top' => true,
				'nodes' => [
					[
						'title' => esc_html__( 'Getting started', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/getting-started/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Settings', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/settings/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Elementor', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/elementor/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'WooCommerce', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/woocommerce/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Global Sections', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/global-sections/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Customization FAQ\'s', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/customization-faqs/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Demos FAQ\'s', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/demos-faq/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Rey Modules', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/rey-modules/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Troubleshooting', 'rey-core' ),
						'href'  => reycore__support_url('kbtopic/troubleshooting/'),
						'new' => true,
					],
				]
			];

			$nodes['faq'] = [
				'title'  => esc_html__( 'FAQ', 'rey-core' ),
				'href'  => reycore__support_url(),
				'meta_title' => esc_html__( 'Frequently asked questions', 'rey-core' ),
				'new' => true,
				'nodes' => [
					[
						'title' => esc_html__( 'How to work faster with Rey?', 'rey-core' ),
						'href'  => reycore__support_url('kb/design-work-faster-with-rey/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'What are Global Sections?', 'rey-core' ),
						'href'  => reycore__support_url('kb/what-exactly-are-global-sections/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'How to add custom CSS?', 'rey-core' ),
						'href'  => reycore__support_url('kb/how-to-add-custom-css/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'How to get started with Elementor?', 'rey-core' ),
						'href'  => reycore__support_url('kb/getting-started-with-elementor/'),
						'new' => true,
					],
					[
						'title' => esc_html__( 'Optimising your website’s speed', 'rey-core' ),
						'href'  => reycore__support_url('kb/optimising-your-websites-speed/'),
						'new' => true,
					],
				]
			];

		}

		$nodes = apply_filters('reycore/admin_bar_menu/nodes', $nodes);

		foreach($nodes as $i => $node){

			$admin_bar->add_node(
				[
					'id'     => $i === 'main' ? $dashboard_id : $dashboard_id . $i,
					'title'  => $node['title'],
					'href'  => $node['href'],
					'parent' => $i === 'main' ? '' : $dashboard_id,
					'meta'   => [
						'title' => isset($node['meta_title']) ? $node['meta_title'] : '',
						'target' => isset($node['new']) ? '_blank' : '',
						'class' => (isset($node['top']) ? 'rey-abQuickMenu-top' : '') . ' ' . (isset($node['class']) ? $node['class'] : ''),
					],
				]
			);

			if( isset($node['nodes']) ){
				foreach ($node['nodes'] as $k => $subnode) {
					$admin_bar->add_node(
						[
							'id'     => $dashboard_id . $i . $k,
							'title'  => $subnode['title'],
							'href'  => $subnode['href'],
							'parent' => $dashboard_id . $i,
							'meta'   => [
								'title' => isset($subnode['meta_title']) ? $subnode['meta_title'] : '',
								'target' => isset($subnode['new']) ? '_blank' : '',
								'class' => (isset($subnode['top']) ? 'rey-abQuickMenu-top' : '') . ' ' . (isset($subnode['class']) ? $subnode['class'] : ''),
							],
						]
					);
				}
			}

		}
	}


	public function register_admin_scripts(){

		// Scripts
		wp_register_script( 'rey-core-admin-script', REY_CORE_URI . 'assets/js/admin.js', ['jquery', 'underscore', 'wp-util', 'rey-ajax'], REY_CORE_VERSION, true );
		wp_localize_script( 'rey-core-admin-script', 'reyCoreAdmin', apply_filters('reycore/admin_script_params', [
			'rey_core_version' => REY_CORE_VERSION,
			'ajax_url'      => admin_url( 'admin-ajax.php' ),
			'ajax_nonce'    => wp_create_nonce('reycore-ajax-verification'),
			'back_btn_text' => esc_html__('Back to List', 'rey-core'),
			'back_btn_url'  => admin_url('edit.php?post_type=rey-global-sections'),
			'is_customizer' => false,
			'strings' => [
				'refresh_demos_error' => esc_html__('Error. Please retry!', 'rey-core'),
				'reloading' => esc_html__('Reloading page!', 'rey-core'),
				'refresh_btn_text' => esc_html__('Refresh Demos List', 'rey-core'),
				'refreshing_btn_text' => esc_html__('Refreshing..', 'rey-core'),
				'migrating_btn_text' => esc_html__('Migrating..', 'rey-core'),
				'help' => esc_html__('Need help?', 'rey-core'),
			],
			'sound_effect' => REY_CORE_URI . 'assets/audio/ding.mp3',
			'customizer_icons' => REY_CORE_URI . 'assets/images/customizer-icons.svg',
			'support_url__post_tax_cat' => reycore__support_url('kb/settings-for-individual-page-post-taxonomy-category/'),
		]) );

		$rtl = reyCoreAssets()::rtl();

		$reycore_styles = [
			'rey-core-admin-style' => [
				'src'     => REY_CORE_URI . 'assets/css/admin'. $rtl .'.css',
				'deps'    => [],
				'version' => REY_CORE_VERSION,
			],
			'reycore-frontend-admin' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/frontend-admin/frontend-admin' . $rtl . '.css',
				'deps'    => [],
				'version' => REY_CORE_VERSION,
			],
		];

		foreach($reycore_styles as $handle => $style ){
			wp_register_style($handle, $style['src'], $style['deps'], $style['version']);
		}
	}

	public function enqueue_admin_scripts(){
		wp_enqueue_script( 'rey-core-admin-script');
		wp_enqueue_style( 'rey-core-admin-style');
	}

	public function enqueue_scripts(){
		if( is_user_logged_in() ){
			wp_enqueue_script(
				'reycore-frontend-admin',
				REY_CORE_URI . 'assets/js/general/c-frontend-admin.js',
				['jquery', 'reycore-scripts'],
				REY_CORE_VERSION,
				true
			);
		}
	}

}
