<?php
namespace ReyCore\Modules\MiniCartBubble;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base extends \ReyCore\Modules\ModuleBase {

	const ASSET_HANDLE = 'reycore-module-minicart-bubble';

	public function __construct()
	{
		add_action( 'reycore/woocommerce/init', [$this, 'init']);
	}

	public function init() {

		if( ! $this->is_enabled() ){
			return;
		}

		add_action( 'reycore/customizer/section=header-mini-cart', [ $this, 'add_customizer_options' ] );
		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_filter( 'theme_mod_header_cart__cross_sells_bubble', [ $this, 'disable_header_cart__cross_sells_bubble' ] );
		add_filter( 'woocommerce_add_to_cart_fragments', [$this, 'cart_fragment'] );
		add_action( 'reycore/woocommerce/minicart/cart_panel', [ $this, 'cart_panel' ] );
	}

	public function register_assets($assets){

		$assets->register_asset('styles', [
			self::ASSET_HANDLE => [
				'src'     => self::get_path( basename( __DIR__ ) ) . '/style.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
			]
		]);

		$assets->register_asset('scripts', [
			self::ASSET_HANDLE => [
				'src'     => self::get_path( basename( __DIR__ ) ) . '/script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
			]
		]);

	}

	public function cart_panel(){

		reyCoreAssets()->add_styles( self::ASSET_HANDLE );
		reyCoreAssets()->add_scripts( self::ASSET_HANDLE );

		if( ! did_action('reycore/woocommerce/ajax/scripts') ){
			do_action('reycore/woocommerce/ajax/scripts');
		}

	}

	public function cross_sells_bubble(){

		$output = '';

		if( ! ($minicart_tag = reycore_wc__get_tag('minicart')) ){
			return $output;
		}

		if( isset($_REQUEST['wc-ajax']) && 'add_to_cart' === reycore__clean($_REQUEST['wc-ajax']) ){
			if( ! (isset($_REQUEST['product_id']) && $product_id = absint($_REQUEST['product_id'])) ) {
				return $output;
			}
		}

		else if( ! (isset($_REQUEST['add-to-cart']) && $product_id = absint($_REQUEST['add-to-cart'])) ){
			return $output;
		}

		$settings = apply_filters('reycore/woocommerce/cartpanel/cross_sells_bubble', [
			'enable' => $this->is_enabled(),
			'limit' => get_theme_mod('header_cart__cross_sells_bubble_limit', 3),
			'rating' => false,
			'title' => get_theme_mod('header_cart__cross_sells_bubble_title', __( 'You may also like&hellip;', 'woocommerce' )),
			'add_to_cart' => esc_html__( 'Continue shopping', 'woocommerce' ),
		]);

		if( ! $settings['enable'] ){
			return $output;
		}

		if( ! ($product = wc_get_product($product_id)) ){
			return $output;
		}

		if( ! ($cs_ids = $product->get_cross_sell_ids()) ){
			return $output;
		}

		if( ! is_array($cs_ids) ){
			return $output;
		}

		$cs_ids = array_unique( $settings['limit'] > 0 ? array_slice( $cs_ids, 0, $settings['limit'] ) : $cs_ids );

		$exclude_items = [];

		foreach ( WC()->cart->get_cart() as $item ) {
			if ( $item['data'] && $pid = $item['data']->get_id() ) {
				$exclude_items[] = $pid;
			}
		}

		if( ! ($cross_sells_data = $minicart_tag->render_cross_sells( $cs_ids, $exclude_items )) ){
			return $output;
		}

		$output .= '<div class="rey-crossSells-bubble" >';
			$output .= sprintf('<h3 class="rey-crossSells-bubble-title">%s</h3>', $settings['title']);
			$output .= $cross_sells_data;
			$output .= sprintf('<div><a class="rey-crossSells-bubble-close btn btn-primary-outline btn--block" href="#">%s</a></div>', $settings['add_to_cart'] );
		$output .= '</div>';

		return $output;
	}

	/**
	 * Cart Fragments
	 * Ensure cart contents update when products are added to the cart via AJAX
	 *
	 * @param  array $fragments Fragments to refresh via AJAX.
	 * @return array            Fragments to refresh via AJAX
	 */
	function cart_fragment( $fragments ) {
		$fragments['rey_cart_cross_sells'] = $this->cross_sells_bubble();
		return $fragments;
	}

	/**
	 * This feature is exclusive to desktops.
	 * No reason to load it on mobiles.
	 */
	function disable_header_cart__cross_sells_bubble($mod){

		if( reycore__is_mobile() && reycore__supports_mobile_caching() ){
			return false;
		}

		return $mod;
	}

	public function add_customizer_options( $section ){

		$section->add_title(  esc_html__('SIDE BUBBLE', 'rey-core') );

		$section->add_control( [
			'type'        => 'toggle',
			'settings'    => 'header_cart__cross_sells_bubble',
			'label'       => esc_html__( 'Enable', 'rey-core' ),
			'description' => esc_html__( 'After a product has been added to cart, a bubble with the products Cross-sells items will be displayed.', 'rey-core' ),
			'default'     => true,
		] );

		$section->add_control( [
			'type'        => 'text',
			'settings'    => 'header_cart__cross_sells_bubble_title',
			'label'       => esc_html__( 'Title', 'rey-core' ),
			'default'     => '',
			'input_attrs'     => [
				'placeholder' => __( 'You may also like&hellip;', 'woocommerce' ),
			],
			'active_callback' => [
				[
					'setting'  => 'header_cart__cross_sells_bubble',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

		$section->add_control( [
			'type'        => 'rey-number',
			'settings'    => 'header_cart__cross_sells_bubble_limit',
			'label'       => esc_html__( 'Products Limit', 'rey-core' ),
			'default'     => 3,
			'choices'     => [
				'min'  => 1,
				'max'  => 20,
				'step' => 1,
			],
			'active_callback' => [
				[
					'setting'  => 'header_cart__cross_sells_bubble',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

	}

	public function is_enabled() {
		return get_theme_mod('header_cart__cross_sells_bubble', true);
	}

	public static function __config(){
		return [
			'id' => basename(__DIR__),
			'title' => esc_html_x('Mini-Cart Bubble popup with Cross-Sells', 'Module name', 'rey-core'),
			'description' => esc_html_x('Shows a popup filled with Cross-sell products, right next to the mini-cart, after a product was added to cart.', 'Module description', 'rey-core'),
			'icon'        => '',
			'categories'  => ['woocommerce'],
			'keywords'    => [''],
			'help'        => '',
		];
	}

	public function module_in_use(){
		return $this->is_enabled();
	}
}
