<?php
namespace ReyCore\Modules\MiniCartCarousel;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base extends \ReyCore\Modules\ModuleBase {

	const ASSET_HANDLE = 'reycore-module-minicart-carousel';

	public function __construct()
	{
		add_action( 'reycore/woocommerce/init', [$this, 'init']);
	}

	public function init() {

		if( ! $this->is_enabled() ){
			return;
		}

		add_action( 'reycore/customizer/section=header-mini-cart', [ $this, 'add_customizer_options' ] );
		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_action( 'reycore/woocommerce/minicart/cart_panel', [ $this, 'cart_panel' ] );
		add_filter( 'theme_mod_header_cart__cross_sells_carousel', [ $this, 'mod_header_cart__cross_sells_carousel' ] );
		add_action( 'reycore/woocommerce/minicart/before_totals', [$this, 'cross_sells_carousel'], 10);
	}

	public function register_assets($assets){

		$assets->register_asset('styles', [
			self::ASSET_HANDLE => [
				'src'     => self::get_path( basename( __DIR__ ) ) . '/style.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
			]
		]);

		$assets->register_asset('scripts', [
			self::ASSET_HANDLE => [
				'src'     => self::get_path( basename( __DIR__ ) ) . '/script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
			]
		]);

	}


	function cross_sells_carousel(){

		if( ! ($minicart_tag = reycore_wc__get_tag('minicart')) ){
			return ;
		}

		$settings = apply_filters('reycore/woocommerce/cartpanel/cross_sells_carousel', [
			'enable' => get_theme_mod('header_cart__cross_sells_carousel', true),
			'limit' => get_theme_mod('header_cart__cross_sells_carousel_limit', 10),
			'title' => get_theme_mod('header_cart__cross_sells_carousel_title', __( 'You may also like&hellip;', 'woocommerce' )),
			'mobile' => get_theme_mod('header_cart__cross_sells_carousel_mobile', true),
			'autoplay' => false,
			'autoplay_duration' => 3000,
		]);

		if( ! $settings['enable'] ){
			return;
		}

		$cross_sells = WC()->cart->get_cross_sells();
		$cross_sells = array_unique( $settings['limit'] > 0 ? array_slice( $cross_sells, 0, $settings['limit'] ) : $cross_sells );

		if( ! ($cross_sells_data = $minicart_tag->render_cross_sells( $cross_sells )) ){
			return;
		}

		$class = 'splide rey-crossSells-carousel';

		if( $settings['mobile'] ){
			$class .= ' --dnone-desktop --dnone-tablet';
		}

		$slider_config = wp_json_encode([
			'autoplay' => $settings['autoplay'],
			'autoplaySpeed' => $settings['autoplay_duration'],
		]);

		$assets_config = reyCoreAssets()->get_assets_paths([
			'styles' => ['rey-wc-general', 'rey-splide'],
			'scripts' => [self::ASSET_HANDLE, 'splidejs', 'rey-splide'],
		]);

		printf('<div class="%1$s" data-slider-config=\'%4$s\' data-assets-config=\'%5$s\'><h3 class="rey-crossSells-carousel-title">%2$s</h3>%3$s</div>',
			$class,
			$settings['title'],
			$cross_sells_data,
			$slider_config,
			!empty($assets_config) ? wp_json_encode($assets_config) : ''
		);
	}


	public function cart_panel(){

		reyCoreAssets()->add_styles( self::ASSET_HANDLE );
		// reyCoreAssets()->add_scripts( self::ASSET_HANDLE );

		if( ! did_action('reycore/woocommerce/ajax/scripts') ){
			do_action('reycore/woocommerce/ajax/scripts');
		}

	}

	public function add_customizer_options( $section ){

		$section->add_title( esc_html__('CROSS-SELLS CAROUSEL', 'rey-core') );

		$section->add_control( [
			'type'        => 'toggle',
			'settings'    => 'header_cart__cross_sells_carousel',
			'label'       => esc_html__( 'Enable Carousel', 'rey-core' ),
			'description' => esc_html__( 'This will display a carousel containing all the cross-sells products linked to the products in the cart.', 'rey-core' ),
			'default'     => true,
		] );

		$section->add_control( [
			'type'        => 'text',
			'settings'    => 'header_cart__cross_sells_carousel_title',
			'label'       => esc_html__( 'Title', 'rey-core' ),
			'default'     => '',
			'input_attrs'     => [
				'placeholder' => __( 'You may also like&hellip;', 'woocommerce' ),
			],
			'active_callback' => [
				[
					'setting'  => 'header_cart__cross_sells_carousel',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

		$section->add_control( [
			'type'        => 'rey-number',
			'settings'    => 'header_cart__cross_sells_carousel_limit',
			'label'       => esc_html__( 'Products Limit', 'rey-core' ),
			'default'     => 10,
			'choices'     => [
				'min'  => 1,
				'max'  => 20,
				'step' => 1,
			],
			'active_callback' => [
				[
					'setting'  => 'header_cart__cross_sells_carousel',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

		$section->add_control( [
			'type'        => 'toggle',
			'settings'    => 'header_cart__cross_sells_carousel_mobile',
			'label'       => esc_html__( 'Show only on Mobile', 'rey-core' ),
			'default'     => true,
			'active_callback' => [
				[
					'setting'  => 'header_cart__cross_sells_carousel',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

	}

	/**
	 * This feature is exclusive to mobile if specified.
	 * No reason to load it on desktop.
	 */
	public function mod_header_cart__cross_sells_carousel($mod){

		if( get_theme_mod('header_cart__cross_sells_carousel_mobile', true) ){
			if( ! reycore__is_mobile() && reycore__supports_mobile_caching() ){
				return false;
			}
		}

		return $mod;
	}

	public function is_enabled() {
		return get_theme_mod('header_cart__cross_sells_carousel', true);
	}

	public static function __config(){
		return [
			'id' => basename(__DIR__),
			'title' => esc_html_x('Mini-Cart Carousel with Cross-Sells', 'Module name', 'rey-core'),
			'description' => esc_html_x('Shows a carousel of products inside the mini-cart, based on added to cart product\'s cross-sell products.', 'Module description', 'rey-core'),
			'icon'        => '',
			'categories'  => ['woocommerce'],
			'keywords'    => [''],
			'help'        => '',
		];
	}

	public function module_in_use(){
		return $this->is_enabled();
	}
}
