<?php
namespace ReyCore\Modules;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class ModuleManager extends \ReyCore\ManagerBase
{

	const DB_OPTION = 'reycore-disabled-modules';

	public $disabled_modules = [];

	public $is_manager = false;

	public function __construct()
	{

		$this->disabled_modules = (array) get_option( self::DB_OPTION, $this->get_default_disabled_items() );

		if( ! is_admin() ){
			return;
		}

		$this->is_manager = (isset($_REQUEST['page']) && $this->get_menu_slug() === reycore__clean($_REQUEST['page']));

		if( ! reycore__get_props('modules_manager') ){
			return;
		}

		parent::__construct();
	}

	public function get_id(){
		return 'modules';
	}

	public function get_menu_title(){
		return __( 'Modules Manager', 'rey-core' );
	}

	public function set_page_config(){
		return [
			'title' => esc_html__('Modules Manager', 'rey-core'),
			'description' => __('Use this tool to control what modules and features can run in your site. Disabling a module will prevent any trace of its code to run.<br>If you\'re not sure if a certain module is actually used inside the site, you can use the Scan function on the right side.', 'rey-core'),
			'singular_item' => esc_html__('Module', 'rey-core'),
			'plural_item' => esc_html__('Modules', 'rey-core'),
			'not_in_use' => esc_html__('Not in use. Safe to deactivate module.', 'rey-core'),
		];
	}

	public function get_default_disabled_items(){
		return apply_filters('reycore/modules/disabled_defaults', [
			'product-grid-custom-tax',
		] );
	}

	public function prepare_items(){

		$categories = [
			'elementor' => 'Elementor',
			'woocommerce' => 'WooCommerce',
			'frontend' => 'Frontend',
			'misc' => 'Miscellaneous',
		];

		$groups = [];

		$modules_instance = \ReyCore\Plugin::instance()->modules;

		foreach ( $modules_instance->get_modules_list() as $module_id => $supported) {

			if( ! $supported ){
				continue;
			}

			// Normalize class name
			$class_name = ucwords( str_replace( '-', ' ', $module_id ) );
			$class_name = str_replace( ' ', '', $class_name );
			$class_name = \ReyCore\Helper::fix_class_name($class_name, 'Modules', 'Base');

			if( ! class_exists($class_name) ){
				trigger_error('Module classname is incorrect and cannot be loaded.', E_USER_NOTICE);
				continue;
			}

			$module = $class_name::get_module_data();

			if( ! $module['show_in_manager'] ){
				continue;
			}

			$item_category = $module['categories'][0];

			$groups[ $item_category ]['category'] = $categories[ $item_category ];
			$groups[ $item_category ]['items'][$module_id] = $module;

		}

		$this->groups = $groups;

	}

	public function ajax__scan_unused( $data ){

		if ( ! current_user_can('install_plugins') ) {
			return [
				'errors' => [ 'Operation not allowed!' ]
			];
		}

		if( ! isset($data['item']) ){
			return [
				'errors' => [ 'Item not passed!' ]
			];
		}

		// if for some reason the module instance can't be retrieved
		// mark it as in use
		if( ! ($module = \ReyCore\Plugin::instance()->modules->get_module($data['item'])) ){
			return true;
		}

		return $module->module_in_use();

	}

	public function get_all_items(){
		return array_keys( \ReyCore\Plugin::instance()->modules->get_modules_list() );
	}

}
